/*
 * Sequoia: Database clustering technology.
 * Copyright (C) 2005-2006 Continuent, Inc.
 * Contact: sequoia@continuent.org
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Initial developer(s): Gilles Rayrat
 * Contributor(s): 
 */

// not to derive: just to get default parameters
#include "../ConnectionSetup.hpp"

#include "Connection.hpp"
#include "Common.hpp"
#include "ConnectionParameters.hpp"
#include "CarobException.hpp"
#include "TestConnect.hpp"

using std::wstring;

using namespace CarobNS;

void TestConnect::setUp()
{
  connectionPtr = NULL;
}

void TestConnect::tearDown()
{
}

void TestConnect::testConnectBadAddress()
{
  const wstring fctName(L"TestConnect::testConnectBadAddress");
  std::vector<ControllerInfo> ctrls;
  try
  {
    logInfo(fctName, L"Creating controller info - should fail");
    ctrls.push_back(ControllerInfo(L"dummyAddress", ConnectionSetup::DEFAULT_PORT1));
    CPPUNIT_ASSERT(false);
  }
  catch (ConnectionException ce)
  {
    logError(fctName, L"Connection failed (this is ok). Exception: "
        + ce.description());
  }
  catch (CarobException unexpected)
  {
    logInfo(fctName, L"Caught other exception: " + unexpected.description());
    // We should receive an exception instead of coming here
    CPPUNIT_ASSERT(false);
  }
}

void TestConnect::testConnectBadPort()
{
  const wstring fctName(L"TestConnect::testConnectBadPort");
  std::vector<ControllerInfo> ctrls;
  ctrls.push_back(ControllerInfo(ConnectionSetup::DEFAULT_HOST1, 12345));
  ConnectionParameters connectionPrms(ctrls,
                                      ConnectionSetup::DEFAULT_DB,
                                      ConnectionSetup::DEFAULT_USER,
                                      ConnectionSetup::DEFAULT_PASSWD);
  logInfo(fctName, L"Connecting to controller - should fail");
  try
  {
    connectionPtr = new Connection(connectionPrms);
    // We should receive an exception instead of coming here
    CPPUNIT_ASSERT(false);
  }
  catch (NoMoreControllerException nmce)
  {
    logError(fctName, L"Connection failed (this is ok). Exception: "
        + nmce.description());
  }
  catch (CarobException unexpected)
  {
    logInfo(fctName, L"Caught other exception: " + unexpected.description());
    // We should receive an exception instead of coming here
    CPPUNIT_ASSERT(false);
  }
}
void TestConnect::testConnectBadDB()
{
  const wstring fctName(L"TestConnect::testConnectBadDB");
  std::vector<ControllerInfo> ctrls;
  ctrls.push_back(ControllerInfo(ConnectionSetup::DEFAULT_HOST1, ConnectionSetup::DEFAULT_PORT1));
  ConnectionParameters connectionPrms(ctrls,
                                      L"dummyDB",
                                      ConnectionSetup::DEFAULT_USER,
                                      ConnectionSetup::DEFAULT_PASSWD);
  try
  {
    connectionPtr = new Connection(connectionPrms);
    logInfo(fctName, L"Connecting to controller - should fail");

    // We should receive an exception instead of coming here
    CPPUNIT_ASSERT(false);
  }
  catch (VirtualDatabaseUnavailableException ae)
  {
    logError(fctName, L"Connection failed (this is ok). Exception: "
        + ae.description());
  }
  catch (CarobException unexpected)
  {
    logInfo(fctName, L"Caught other exception: " + unexpected.description());
    // We should receive an exception instead of coming here
    CPPUNIT_ASSERT(false);
  }
}

void TestConnect::testConnectBadUser()
{
  const wstring fctName(L"TestConnect::testConnectBadUser");
  std::vector<ControllerInfo> ctrls;
  ctrls.push_back(ControllerInfo(ConnectionSetup::DEFAULT_HOST1, ConnectionSetup::DEFAULT_PORT1));
  ConnectionParameters connectionPrms(ctrls,
                                      ConnectionSetup::DEFAULT_DB,
                                      L"dummyuser",
                                      ConnectionSetup::DEFAULT_PASSWD);
  try
  {
    connectionPtr = new Connection(connectionPrms);
    logInfo(fctName, L"Connecting to controller - should fail");
    
    // We should receive an exception instead of coming here
    CPPUNIT_ASSERT(false);
  }
  catch (AuthenticationException ae)
  {
    logError(fctName, L"Connection failed (this is ok). Exception: "
        + ae.description());
  }
  catch (CarobException unexpected)
  {
    logInfo(fctName, L"Caught other exception: " + unexpected.description());
    // We should receive an exception instead of coming here
    CPPUNIT_ASSERT(false);
  }
}

void TestConnect::testConnectBadPass()
{
  const wstring fctName(L"TestConnect::testConnectBadPass");
  std::vector<ControllerInfo> ctrls;
  ctrls.push_back(ControllerInfo(ConnectionSetup::DEFAULT_HOST1, ConnectionSetup::DEFAULT_PORT1));
  ConnectionParameters connectionPrms(ctrls,
                                      ConnectionSetup::DEFAULT_DB,
                                      ConnectionSetup::DEFAULT_USER,
                                      L"dummy");
  try
  {
    connectionPtr = new Connection(connectionPrms);
    logInfo(fctName, L"Connecting to controller - should fail");
    
    // We should receive an exception instead of coming here
    CPPUNIT_ASSERT(false);
  }
  catch (AuthenticationException ae)
  {
    logError(fctName, L"Connection failed (this is ok). Exception: "
        + ae.description());
  }
  catch (CarobException unexpected)
  {
    logInfo(fctName, L"Caught other exception: " + unexpected.description());
    // We should receive an exception instead of coming here
    CPPUNIT_ASSERT(false);
  }
}

void TestConnect::testConnectGood()
{
  const wstring fctName(L"TestConnect::testConnectGood");
  std::vector<ControllerInfo> ctrls;
  ctrls.push_back(ControllerInfo(ConnectionSetup::DEFAULT_HOST1, ConnectionSetup::DEFAULT_PORT1));
  ConnectionParameters connectionPrms(ctrls,
                                      ConnectionSetup::DEFAULT_DB,
                                      ConnectionSetup::DEFAULT_USER,
                                      ConnectionSetup::DEFAULT_PASSWD);
  logInfo(fctName, L"Connecting to controller - this should succeed");
  connectionPtr = new Connection(connectionPrms);
  logInfo(fctName, L"Connection succeeded");
  delete connectionPtr;
}

CppUnit::Test* TestConnect::suite()
{
  CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite("10.TestConnect");
  suiteOfTests->addTest(new CppUnit::TestCaller<TestConnect>(
                                 "10.TestConnect::testConnectBadAddress",
                                 &TestConnect::testConnectBadAddress));
  suiteOfTests->addTest(new CppUnit::TestCaller<TestConnect>(
                                 "10.TestConnect::testConnectBadPort",
                                 &TestConnect::testConnectBadPort));
  suiteOfTests->addTest(new CppUnit::TestCaller<TestConnect>(
                                 "10.TestConnect::testConnectBadDB", 
                                 &TestConnect::testConnectBadDB));
  suiteOfTests->addTest(new CppUnit::TestCaller<TestConnect>(
                                 "10.TestConnect::testConnectBadUser", 
                                 &TestConnect::testConnectBadUser));
  suiteOfTests->addTest(new CppUnit::TestCaller<TestConnect>(
                                 "10.TestConnect::testConnectBadPass", 
                                 &TestConnect::testConnectBadPass));
  suiteOfTests->addTest(new CppUnit::TestCaller<TestConnect>(
                                 "10.TestConnect::testConnectGood", 
                                 &TestConnect::testConnectGood));

  return suiteOfTests;
}
