/**
 * Sequoia: Database clustering technology.
 * Copyright (C) 2005-2006 Continuent, Inc.
 * Contact: sequoia@continuent.org
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Initial developer(s): Marc Herbert
 * Contributor(s): 
 */

#include "Common.hpp"
#include "CarobException.hpp"
#include "DriverResultSet.hpp"
#include "Statement.hpp"


#include "TestSimpleUnicode.hpp"

#define TABLE_NAME L"unicodebasic"

#define DEFAULT_SQL_STRING_TYPE L"VARCHAR(200)"
#define TABLE_TYPE "(teststring " DEFAULT_SQL_STRING_TYPE L")"


using namespace CarobNS;

using std::wstring;



void writeRead(Connection *connectionPtr, const std::wstring& fctName,
               const std::wstring& testString)
{

    if (isInfoEnabled())
    {
        logInfo(fctName, L"Executing replace table " TABLE_NAME L" " TABLE_TYPE);
    }

    createOrReplaceTable(connectionPtr, TABLE_NAME, TABLE_TYPE);

    Statement *stmt = connectionPtr->createStatement();
    
    wstring insertcmd(L"INSERT INTO " TABLE_NAME " values ('");
    insertcmd += testString;
    insertcmd += L"')";

    int updatecount = stmt->executeUpdate(insertcmd);
    CPPUNIT_ASSERT(1 == updatecount);

    DriverResultSet* drsPtr = stmt->executeQuery(L"SELECT * FROM " TABLE_NAME);
    CPPUNIT_ASSERT(drsPtr->next());

    wstring returnedString = drsPtr->getString(1);
    CPPUNIT_ASSERT(0 == testString.compare(returnedString));

    CPPUNIT_ASSERT(!drsPtr->next());

    connectionPtr->deleteStatement(stmt);
}

void TestSimpleUnicode::testPBK()
{
    const wstring fctName(L"TestSimpleUnicode::testPBK");
    // PBK
    // phi beta kappa + zero + last character (kappa)

    wchar_t test_[] = { 0x03c6, 0x03b2, 0x03ba };
    wstring test(test_, 3);

    writeRead(connectionPtr, fctName, test);
}


// warning: some DBMS (e.g., PostgreSQL) support only C-style, Null Terminated Strings.
// This test WILL FAIL with them. (SQLstate 08P01 for PostgreSQL)

void TestSimpleUnicode::testPBKNull()
{
    const wstring fctName(L"TestSimpleUnicode::testPBKNull");
    // PBK
    // phi beta kappa + zero + last character (kappa) again
    // this is is even more evil because of the zero (see SEQUOIA-133)
    wchar_t test_[] = { 0x03c6, 0x03b2, 0x03ba, 0, 0x03ba };
    wstring test(test_, 5);

    writeRead(connectionPtr, fctName, test);
}


void TestSimpleUnicode::testLatin1CodeSubset()
{
    const wstring fctName(L"TestSimpleUnicode::testLatin1CodeSubset");

    wchar_t test_[] = { 307, 340, 240, 241 };
    wstring test(test_, 4);

    writeRead(connectionPtr, fctName, test);

}

// CAROB-101
void TestSimpleUnicode::testInvalidUnicode()
{
    const wstring fctName(L"TestSimpleUnicode::testInvalidUnicode");

    
    wchar_t invalid_ [] = { 0x110000 };
    wstring invalid_unicode(invalid_, 1);

    try {
        writeRead(connectionPtr, fctName, invalid_unicode);
        CPPUNIT_FAIL("should not be successful to write/read invalid unicode");
    } catch (const CarobException& ce) {
        // TODO: check that this is an expected exception like
        //  java.nio.charset.UnmappableCharacterException
        // or alike.
        // If we get something totally unrelated instead (like
        // for instance "all controllers down") => rethrow

         // to workaround a buggy IOException, leave time for the
         // pinger to put the controller back into the live pool
        sleep(2);

        if (ce.getSQLState() != L"Unmappablecharacter")
            throw;
    }
}

// List of tests in this class
CppUnit::Test* TestSimpleUnicode::suite()
{
    CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite("30.TestSimpleUnicode");

    suiteOfTests->addTest(new CppUnit::TestCaller<TestSimpleUnicode>(
                              "30.TestSimpleUnicode::testPBK",
                              &TestSimpleUnicode::testPBK));

// ? : TODO: check why mysql throws errors
// MH: Probably because it refuses inline nulls?
// #ifndef CAROB_TEST_USE_MYSQL
    suiteOfTests->addTest(new CppUnit::TestCaller<TestSimpleUnicode>(
                              "30.TestSimpleUnicode::testPBKNull",
                              &TestSimpleUnicode::testPBKNull));
// #endif

    suiteOfTests->addTest(new CppUnit::TestCaller<TestSimpleUnicode>(
                              "30.TestSimpleUnicode::testLatin1CodeSubset",
                              &TestSimpleUnicode::testLatin1CodeSubset));

    suiteOfTests->addTest(new CppUnit::TestCaller<TestSimpleUnicode>(
                              "30.TestSimpleUnicode::testInvalidUnicode",
                              &TestSimpleUnicode::testInvalidUnicode));

    return suiteOfTests;
}


/*
 * Local Variables:
 * c-file-style: "bsd"
 * c-basic-offset: 4
 * indent-tabs-mode: nil
 * End:
 */
