/*
 * Sequoia: Database clustering technology.
 * Copyright (C) 2005-2006 Continuent, Inc.
 * Contact: sequoia@continuent.org
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Initial developer(s): Marc Herbert
 * Contributor(s): 
 */

#include "TestIEEE754.hpp"

#include "ParameterStatement.hpp"

#include <cmath>

#include <limits>

#include "C99Math.h"

// substitute here with the float32_t and float64_t types of your
// favorite DBMS
#define TABLE_TYPE L"(i int, f real, d double precision)"

#define TABLE_NAME L"realdouble"

#define INSERT_STMT L"insert into " TABLE_NAME L" values (?, ?, ?)"
#define RETRIEVE_STMT L"select * from " TABLE_NAME

#if !defined __STDC_IEC_559__ && !defined __MINGW32__
#warning only IEEE 754 platforms are supported
#endif


#include <iostream>


#ifdef CAROB_DEBUG_FLOAT
namespace CarobNS {
uint32_t F2I(float f);
uint64_t D2I(double d);
}
#define floatToU32Bits(f) F2I(f)
#define doubleToU64Bits(d) D2I(d)
#endif

using namespace CarobNS;

namespace {

    typedef std::numeric_limits<float> Floats;
    typedef std::numeric_limits<double> Doubles;
    typedef struct _fd { float f; double d; } fd_t;


void
printrow(float f, double d)
{
    using std::cout;

    std::ios_base::fmtflags old_flags = cout.flags();

    cout.setf(std::ios_base::scientific, std::ios_base::floatfield);
    cout.setf(std::ios_base::hex, std::ios_base::basefield);
    cout.setf(std::ios_base::showbase | std::ios_base::internal);
    cout.fill('0');

    cout.precision(9);
    cout << f << " (=";
    cout.width(10);
#ifdef CAROB_DEBUG_FLOAT // see SQLDataSerialization.cpp 
    cout << floatToU32Bits(f) << ") | ";
#endif
    cout.precision(17);
    cout << d << " (=";
    cout.width(18);
#ifdef CAROB_DEBUG_FLOAT
    cout << doubleToU64Bits(d) << ")";
#endif

    // restore settings
    cout.flags(old_flags);

}

template <class FP> bool
better_equal(FP a, FP b)
{
    if (isnan(a)) // isnan() is C99 but not C++98
        return (isnan(b) && (signbit(a) == signbit(b)));
    else
        return (a == b);
}


void
write(Connection * conn, const fd_t data[])
{
    // WRITE
    ParameterStatement *
        insert(conn->
               createParameterStatement(INSERT_STMT));

    int row = 0; 
    while (data[row].f || data[row].d) {
        insert->setInt(1, row);
        insert->setFloat(2, data[row].f);
        insert->setDouble(3, data[row].d);
        int uc =
            insert->executeUpdate();
        CPPUNIT_ASSERT(1 == uc);
        row++;
    }
    conn->deleteStatement(insert);
}

void
read_compare(Connection * conn, const fd_t data[])
{
    // READ
    Statement *
        retrieve(conn->
                 createStatement());

    DriverResultSet* drsPtr = 
        retrieve->executeQuery(RETRIEVE_STMT);

    int row = 0;
    while (data[row].f || data[row].d) {
        CPPUNIT_ASSERT(drsPtr->next());
        CPPUNIT_ASSERT(drsPtr->getAsInt(1) == row);

#ifdef CAROB_DEBUG_FLOAT
        std::cout << "tried to send: ";
        printrow(data[row].f, data[row].d);
        std::cout << std::endl << "received:      ";
        printrow(drsPtr->getAsFloat(2), drsPtr->getDouble(3));
        std::cout << std::endl;
        // asserting may interrupt test
#else // assert it
        CPPUNIT_ASSERT(better_equal(drsPtr->getAsFloat(2), data[row].f));
        CPPUNIT_ASSERT(better_equal(drsPtr->getDouble(3), data[row].d));
#endif
        row++;
    }
    conn->deleteStatement(retrieve);
}

} // unnamed namespace 

// send inline string to avoid setFloat()
void
TestIEEE754::testSimpleReceiveOnly()
{
    createOrReplaceTable(connectionPtr, TABLE_NAME, TABLE_TYPE);

    // warning: it must be possible to write these in binary without any rounding!
    fd_t basicdata[] =
        { { 0.0, 1.0 },
          { 0, 0 } // END OF DATA
        };

    ParameterStatement *
        insert(connectionPtr->
               createParameterStatement(INSERT_STMT));

    insert->setInt(1, 0);
    // manual setString() to avoid setFloat()
    insert->setString(2, std::wstring(L"0.0"));
    insert->setString(3, std::wstring(L"1.0"));
    int uc = insert->executeUpdate();
    CPPUNIT_ASSERT(1 == uc);


    read_compare(connectionPtr, basicdata);
    connectionPtr->deleteStatement(insert);

}


void
TestIEEE754::testBasic()
{
    createOrReplaceTable(connectionPtr, TABLE_NAME, TABLE_TYPE);

    fd_t basicdata[] =
        { { 0.0, 1 },
          { 1, 0.0 },
          { -1, -1 },
          { -5e30, 1e-200 },
          { -0.1, 0.1 },
          { -1.0/3, 1.0/3 },
          { Floats::round_error(), Doubles::round_error() },
           { Floats::epsilon(), -Doubles::epsilon() },
          {  -Floats::min()*2, Doubles::min()*2 },
          { Floats::max()/2, -Doubles::max()/2 },
          {  -Floats::min(), Doubles::min() },
          { Floats::max(), -Doubles::max() },
          { -Floats::denorm_min(), Doubles::denorm_min() },
          { -Floats::min(), Doubles::min() },
          { Floats::max(), -Doubles::max() },
          { 0, 0 } // END OF DATA
        };

    write(connectionPtr, basicdata);
    read_compare(connectionPtr, basicdata);
  
}

void
TestIEEE754::testInfNaN()
{

    createOrReplaceTable(connectionPtr, TABLE_NAME, TABLE_TYPE);

    fd_t basicdata[] =
        {
            { Floats::infinity(), -Doubles::infinity() },

            // No one seems to make the difference between quiet vs signaling
            { Floats::quiet_NaN(), Doubles::quiet_NaN() },
            { Floats::signaling_NaN(), Doubles::signaling_NaN() },

            // disabled cause java.lang.FloatingDecimal#readJavaFormatString()
            // ignores negative sign of NaN
#if 0
            { -Floats::quiet_NaN(), -Doubles::quiet_NaN() },
            { -Floats::signaling_NaN(), -Doubles::signaling_NaN() },
#endif

            { 0, 0 } // END OF DATA
        };

    write(connectionPtr, basicdata);
    read_compare(connectionPtr, basicdata);

}

CppUnit::Test*
TestIEEE754::suite()
{
    CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite("40.TestIEEE754");
// TODO: check why mysql throws errors
#ifndef CAROB_TEST_USE_MYSQL
    suiteOfTests->addTest(new CppUnit::TestCaller<TestIEEE754>(
                              "40.TestIEEE754::testSimpleReceiveOnly", 
                              &TestIEEE754::testSimpleReceiveOnly));
    suiteOfTests->addTest(new CppUnit::TestCaller<TestIEEE754>(
                              "40.TestIEEE754::testBasic", 
                              &TestIEEE754::testBasic));
    suiteOfTests->addTest(new CppUnit::TestCaller<TestIEEE754>(
                              "40.TestIEEE754::testInfNaN", 
                              &TestIEEE754::testInfNaN));
#endif
    return suiteOfTests;
}


/*
 * Local Variables:
 * c-file-style: "bsd"
 * c-basic-offset: 4
 * indent-tabs-mode: nil
 * End:
 */
